%%%%%%%%%
%%5. get_KCcodes
%	-input: MCS.mat, KCcopies.mat
%	-filters: D,E,F,G,H
%	-output: KCcodes.mat
%%%%%%%%%%
%%[3] >4 OBESE AND IBD SAMPLES
%%[2] >4 IBD SAMPLES
%%[1] >4 OBESE SAMPLES
%%[0] MEASURABLE KCS
%%[-1]NO VAL: KO IS NOT PRESENT IN A CLUSTER
%%[-2]NO VAR: CLUSTER IS PRESENT IN LESS THAN 10 SAMPLE
%%[-3]NO VAL: KCcopies all < .5
%%[-4]NONMAPPABLE KO, ERROR RATE >.5
%%%%%%%%%%%%%%%%%%
clear all
%% SET FILTERS
filter_unmappable_genomes=1; %C [0|1]
filter_unmappable_KOs=1; %D DONT CHANGE
markerCov_thresh=1; %E [1|2|5]
markerCopy_thresh=12;
detectableKO_thresh=.5; %F[0|.5|1]
KCvarClusters_thresh= 10; %G DONT CHANGE
KCobClusters_thresh=[5;5]; %H DONT CHANGE

%% LOAD VARS
load( '../KCScopies/KCScopies.mat');
load '../info/markerKOs_13.mat'
load('../MCS/MCS.mat')
markerCov=MCS.mcs; clear MCS
run('../info/LANDSCAPE_refGenomes_merged_proteins_annotated_kolist_tbl.m');
load '../info/clusterMap.mat'
unmappableKOs=importdata('../info/filterKOs.txt');

%% MAKE KCcodes matrix
KCcodes = zeros(length(KCScopies.clusters),length(KCScopies.kos));

%% get detectableClusters (101x109)
markerCopies(:,:)=sum(KCScopies.vals(:,ismember(KCScopies.kos,markers),:)>detectableKO_thresh,2);

CScodes=(markerCov>=markerCov_thresh) & (markerCopies>=markerCopy_thresh);
nofClustersPerSample=sum(CScodes);
nofSamplesPerCluster=sum(CScodes');

%% G-H. MARK NON-ANALYZABLE CLUSTERS
 obese= strcmp(KCScopies.sampleClass(:,1),'obese') & strcmp(KCScopies.sampleClass(:,2),'healthy');
 healthy= strcmp(KCScopies.sampleClass(:,1),'lean') & strcmp(KCScopies.sampleClass(:,2),'healthy');
 ibd= strcmp(KCScopies.sampleClass(:,2),'IBD') & strcmp(KCScopies.sampleClass(:,1),'lean');
 
obClusters=sum(CScodes(:,obese)')>=KCobClusters_thresh(1) & sum(CScodes(:,healthy)')>=KCobClusters_thresh(2);
ibdClusters=sum(CScodes(:,ibd)')>=KCobClusters_thresh(1) & sum(CScodes(:,healthy)')>=KCobClusters_thresh(2);
varClusters=sum(CScodes(:,obese|healthy|ibd)')>=KCvarClusters_thresh;
KCcodes(~varClusters,:)=-2;
KCcodes(obClusters,:)=1;
KCcodes(ibdClusters,:)=2;
KCcodes(obClusters & ibdClusters,:)=3;



%% MARK KOs ABSENT FROM EACH CLUSTER
[c ia ib] =intersect(KCScopies.kos,items);
ix(ia)=ib;
pptable=pptable(:,ix);
for ii=1:length(KCScopies.clusters)
    taxIDs=clusterMap(strmatch(KCScopies.clusters{ii},clusterMap(:,2),'exact'),1);
    cluster2kos(ii,:)=sum(pptable(ismember(spcs,taxIDs),:),1);
end
KCcodes(cluster2kos<1)=-1;

%% D. MARK NON_MAPPABLE KOS
if filter_unmappable_KOs
    KCcodes(:,ismember(KCScopies.kos,unmappableKOs))=-3;
end

%% F. MARK NON_DETECTABLE KCs 
for ii=1:length(KCScopies.clusters)
    for jj=1:1:length(KCScopies.kos)
%        if ((KCcodes(ii,jj)==0) || (KCcodes(ii,jj)==-3)) && max(KCScopies.vals(ii,jj,:))<detectableKO_thresh
        if (KCcodes(ii,jj)>=0) && (max(KCScopies.vals(ii,jj,CScodes(ii,:)))<detectableKO_thresh)
            KCcodes(ii,jj)= -4;
        end
    end
end

%% SAVE KCcodes
%save([inDIR '/KCcodes_D' num2str(filter_unmappable_KOs) '_E' num2str(markerCov_thresh) '-' num2str(markerCopy_thresh) '_F' num2str(detectableKO_thresh) '.mat'],'KCcodes');
%save([inDIR '/CScodes_D' num2str(filter_unmappable_KOs) '_E' num2str(markerCov_thresh) '-' num2str(markerCopy_thresh) '_F' num2str(detectableKO_thresh) '.mat'],'CScodes');
