%%%%%%%%%%%%%%%%%%%%
%6.get_KCvar
%	-input: KCScopies.mat, KCcodes.mat
%	-filters:I
%	-output: KCvar.mat
%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
close all

%% SET PARAMETERS
stdFactor = 2;
pctSamples = .1;

%% LOAD VARS
load('../dataVars/KCScopies.mat');
load '../dataVars/KCcodes.mat';
load '../dataVars/CScodes_D1_E1-12_F0.5.mat';
load '../dataVars/CLUSTER.mat';

%% initialize KCvar
KCvar.clusters=KCScopies.clusters;
KCvar.kos=KCScopies.kos;
KCvar.koNames=KCScopies.koNames;
KCvar.devthresh = 0;
KCvar.KCmad=KCcodes;
KCvar.Sgroup=zeros(length(KCvar.clusters),length(KCvar.kos),length(KCScopies.samples));
KCvar.sigvar=zeros(length(KCvar.clusters),length(KCvar.kos),4);

    
%% FIND THE MAD FOR EACH KC
   varClusters = sum(KCcodes'>=0)>0;
for cc=find(varClusters) %for each cluster with 10 samples
   clear markerKCS KCvals
   sampleMask=CScodes(cc,:)';  %find samples in which cluster is detectable
   kMask=(KCcodes(cc,:)>=0); 

   for kk=find(kMask) % for each KO in cluster
        KCmedian = median(KCScopies.vals(cc,kk,sampleMask)); %% get median for each KC
        Sdev= abs(KCScopies.vals(cc,kk,sampleMask) - KCmedian); %% get deviation from median for each KCS
        KCvar.KCmad(cc,kk) = median(Sdev); %% get MAD for each KC
        %KCvar.KCmad(cc,kk) = mean(Sdev);
   end
end

%% FIND THE THRESHOLD MAD
KCmad = KCvar.KCmad(KCcodes>=0);
KCvar.devthresh = mean(KCmad(:)) + stdFactor*std(KCmad(:));

%% FIND KCs WITH MAD>THRESH
KCvar.sigvar=KCvar.KCmad>KCvar.devthresh;

%% FIND SAMPLES WITH DEV > THRESH
for cc=find(varClusters) %for each cluster with 10 samples
   clear markerKCS KCvals
   sampleMask=CScodes(cc,:)';  %find samples in which cluster is detectable
   kMask=(KCcodes(cc,:)>=0); 

   for kk=find(kMask) % for each KO in cluster
        KCmedian = median(KCScopies.vals(cc,kk,sampleMask)); %% get median for each KC
        Sdev= KCScopies.vals(cc,kk,sampleMask) - KCmedian; %% get deviation from median for each KCS
        KCvar.Sgroup(cc,kk,sampleMask) = double(Sdev>KCvar.devthresh) - double(Sdev<(-1*KCvar.devthresh)); %% get MAD for each KC
   end
end

%% FIND KCs WITH >10% OF SAMPLES ABOVE/BELOW THRESH
KCvar.sigvar(:,:,2) = sum(KCvar.Sgroup==1,3)> pctSamples.*repmat(sum(CScodes')',1,length(KCvar.kos));
KCvar.sigvar(:,:,3) = sum(KCvar.Sgroup==-1,3)> pctSamples.*repmat(sum(CScodes')',1,length(KCvar.kos));
KCvar.sigvar(:,:,4) = KCvar.sigvar(:,:,2) & KCvar.sigvar(:,:,3) ;

%% MAKE TABLE OF COUNTS
sigSum(:,:)=sum(KCvar.sigvar(varClusters,:,:),2);
allsum=sum(KCcodes(varClusters,:)'>=0);
TABLE=[CLUSTER.names(varClusters) num2cell([sigSum sigSum./repmat(allsum',1,4)])];