#!/usr/bin/perl -w

## merges individual fastas into one multi-entry file, according to a supplied list of files for a specific project

### read in a list of taxIDs
### unzip corresponding file(keep original) and open each fasta (may be just 1 or >1),
### make sure its not a plasmid
### reformat header to match: [taxID_contigID_contigAccession; optional: originalFile, description]
### add sequence to merged file
### remove each unzipped file

$num_args = $#ARGV + 1;
if ($num_args < 2)
{
  print "\nUsage: LANDSCAPE_merge_project_fastas.pl taxids seqDir\n";
  exit;
}

$f_taxids = shift @ARGV;
$seqDir = shift @ARGV;

################################
##OPEN FILES
#############################
open (FH_FNA, ">$f_taxids.contigs.fna") or die "Cannot open $f_taxids.contigs.fna for writing.\n";
open (FH_FNA_HEADERS, ">$f_taxids.contigs.fna.headers") or die "Cannot open $f_taxids.contigs.fna.headers for writing.\n";
open (FH_FAA, ">$f_taxids.proteins.faa") or die "Cannot open $f_taxids.proteins.faa for writing.\n";
open (FH_FAA_HEADERS, ">$f_taxids.proteins.faa.headers") or die "Cannot open $f_taxids.proteins.faa.headers for writing.\n";

open (FH_BED, ">$f_taxids.genes.BED") or die "Cannot open $f_taxids.genes.BED for writing.\n";

open (FH_LOG, ">$f_taxids.mergedSeqs.log") or die "Cannot open logfile for writing.\n";

############################################
##### --- --- Get a list of desired species--- --- ######
$nof_taxids = 0;
print "getting records for: \n";
%h_taxids=();
open (FH, "$f_taxids") or die "Cannot open $f_taxids for reading.\n";
while(<FH>) {
	$line=$_; chomp($line);
	$h_taxid_nofContigs{$line}=0;
	$h_taxid_nofProteins{$line}=0;
	print "$line|";
	$nof_taxids++;
}
close(FH);
print "\n";

	# --- --- Find file corresponding to each taxID, unzip and open each fasta --- --- #
$nofContigs=0;
$nofProteins=0;

foreach $taxid (sort keys %h_taxid_nofContigs)
{
	print "now working on... $taxid\n";
	###########################
	## UNZIP ALL FILES FOR TAXID
	###########################
	@files = glob("$seqDir/nucleotide_fastas/$taxid*.fna.tgz" );
	foreach $file (@files)
	{	`tar xzf $file -C $seqDir/unzipped`;	}
	@files = glob("$seqDir/gene_calls/$taxid" . "_*.ptt.tgz" );
	foreach $file (@files)
	{	`tar xzf $file -C $seqDir/unzipped`;	}
	@files = glob("$seqDir/protein_fastas/$taxid" . "_*.faa.tgz" );
	foreach $file (@files)
	{	`tar xzf $file -C $seqDir/unzipped`;	}

	#############################
	## FOR EACH FNA FILE, ADD SEQ TO MERGED CONTIGS
	################################
	@fnas = glob("$seqDir/unzipped/*.fna");
	foreach $fna (@fnas)
	{
		open (FH, $fna) or die "Cannot open $fna for reading.\n";
		while (<FH>)
		{
			$line = $_; chomp $line;
			if (substr($line,0,1) eq ">")
			{	$header = $line;
				if ($header =~ m/plasmid/)
				{	$get_seq=0;	}
				else
				{	$get_seq=1;
					$nofContigs++; $h_taxid_nofContigs{$taxid}++;
					($acc) = ($fna =~ m/([^\\\/]+).fna$/);
					$replace=$taxid . "_";
					$acc =~ s/$replace//;
					$new_header = ">" . $taxid . "_" .  $h_taxid_nofContigs{$taxid}  . "_" .  $acc . " " . substr($header, 1, length($header)-1);
					print FH_FNA "$new_header\n";
					print FH_FNA_HEADERS "$new_header\n";
				}
			}
			else
			{
				if ($get_seq)
				{	print FH_FNA "$line\n";	}
			}
		}
		close(FH);
		system('rm',$fna);

		######################
		##GET GENE INFO FROM PTT AND MAKE BED FILE
		####################
		($ptt = $fna) =~ s/fna/ptt/;
		$no_genes=0;
		open (PTT, $ptt) or $no_genes=1;
		if (! $no_genes)
		{
			$line=<PTT>;	#print "$line\n";
			$line=<PTT>;  chomp($line);
			($nof_orfs) = ($line =~ m/^(\S+)/);
			<PTT>;
			while(<PTT>)
			{
				$line=$_; chomp($line);
				($loc, $strand, $length, $pid, $gene, $synonym, $code, $cog, $product) = split(/\t/,$line);
				($start,$end) = ($loc =~ m/(.+)\.\.(.+)/);
				$h_taxid_nofProteins{$taxid}++; $nofProteins++;
				$myGeneID = "G" . $taxid . "-" . $h_taxid_nofProteins{$taxid};
				$geneHeader = $myGeneID	. "_C" . $h_taxid_nofContigs{$taxid} . "_" . $start . "_" . $end . " length:$length pid:$pid $synonym $product";
				$contig_header = $taxid . "_" .  $h_taxid_nofContigs{$taxid}  . "_" . $acc;
				$h_pid2header{$pid}=$geneHeader;
				print FH_BED "$contig_header\t$start\t$end\t$myGeneID\t$length\t$strand\t$pid\t$synonym\t$product\n";
			}
			system('rm',$ptt);
			####################################
			## GET PROTEIN SEQS FROM FAA AND LABEL WITH CORRECT GENE HEADER
			####################################
			($faa = $fna) =~ s/fna/faa/;
			open (FAA, $faa) or $no_genes=1;
			while (<FAA>)
			{	$line = $_; chomp $line;
				if (substr($line,0,1) eq ">")
				{	@fields = split(/\|/,$line);
					$pid = $fields[1];
					if (exists $h_pid2header{$pid})
					{	$get_seq=1;
						print FH_FAA ">$h_pid2header{$pid}\n";
						print FH_FAA_HEADERS "$h_pid2header{$pid}\n";
						$h_pid2header{$pid}=0;
					}
					else
					{	print FH_LOG "unmatched protein in faa:$line\n";  $get_seq=0;}
				}
				else
				{
					if ($get_seq)
					{	print FH_FAA "$line\n";	}
				}
			}
			close(FAA);
			system('rm',$faa);

			#######################
			## MAKE SURE ALL THE PROTEINS IN THE PTT FILE WERE FOUND
			#########################
			foreach $pid (sort keys %h_pid2header)
			{	if ($h_pid2header{$pid}>0)
				{	print FH_LOG "unmatched protein in ptt:$pid\n";	}
			}
		}
		else
		{	print FH_LOG "no genes for $fna!!\n";	}
	}
}
	## PRINT COUNTS
print FH_LOG "************************************\n";
print FH_LOG "$nofContigs total contigs merged! \n";
print FH_LOG "nof contigs merged for each taxID:\n";
print  "************************************\n";
print  "$nofContigs total contigs merged! \n";
print  "nof contigs merged for each taxID:\n";
foreach $taxid (sort keys %h_taxid_nofContigs)
{	print FH_LOG "$taxid:$h_taxid_nofContigs{$taxid}|";
	print "$taxid:$h_taxid_nofContigs{$taxid}|";
}

print FH_LOG "************************************\n";
print FH_LOG "$nofProteins total Proteins merged! \n";
print FH_LOG "nof Proteins merged for each taxID:\n";
print  "************************************\n";
print  "$nofProteins total Proteins merged! \n";
print  "nof Proteins merged for each taxID:\n";
foreach $taxid (sort keys %h_taxid_nofProteins)
{	print FH_LOG "$taxid:$h_taxid_nofProteins{$taxid}|";
	print "$taxid:$h_taxid_nofProteins{$taxid}|";
}

close(FH_FNA);
close(FH_FNA_HEADERS);
close(FH_FAA);
close(FH_FAA_HEADERS);
close(FH_LOG);