# batch job script (LANDSCAPE_metahit_bwa.sh)
#
# Use the bash shell to interpret this job script
#$ -S /bin/bash
#
# Remove one # to send an e-mail to the address 
# specified in .sge_request when this job ends.
#$ -m e
#
# Remove one # to only submit this job to nodes
# that have at least 8GB of RAM free.
#$ -l mem_free=8.0G
#$ -pe serial 4-4
GSITSHOST=`/bin/hostname`
GSITSPWD=`/bin/pwd`
GSITSDATE=`/bin/date`
echo "**** JOB STARTED ON $GSITSHOST AT $GSITSDATE"
echo "**** JOB RUNNING IN $GSITSPWD"
echo "----------------------------------------------"
LAB_PATH="/net/borenstein/vol1/PROJECTS/StrainVariation_SG/CODE/perl/3_MAP_SAMPLES"
sampleSet=mockCommunity  #change this to switch between different sample sets: metahit,diabetes,hmp

bedFile=$(echo $(sed -n '6{p;q;}' $LAB_PATH/$sampleSet\_parameters.txt | grep -Po '\S+$'))
sample=$(cat $LAB_PATH/$sampleSet\_bwa_batchFiles/$sampleSet\_bwa.$SGE_TASK_ID | grep -Po '^\S+')
bamDir=$(echo $(sed -n '5{p;q;}' $LAB_PATH/$sampleSet\_parameters.txt | grep -Po '\S+$'))
bamFile=$bamDir/$sample.bam

echo $sample ":calculating stats for $bamFile..."
nofReads=$(Programs/samtools-0.1.18/samtools view -c $bamFile)  #total reads
mReads=$(Programs/samtools-0.1.18/samtools view -c -F 4 $bamFile) #mapped reads
umReads=$(Programs/samtools-0.1.18/samtools view -c -f 4 $bamFile) #unmapped reads
stats=$(
Programs/samtools-0.1.18/samtools view -hF 4 $bamFile | perl perl/SingleGenomes/LANDSCAPE/TESTBWA2h_get_mapped_reads.pl 5 | Programs/samtools-0.1.18/samtools view -bS - | Programs/bedtools-2.17.0/bin/intersectBed -abam stdin -b $bedFile -bed -wao | perl perl/SingleGenomes/LANDSCAPE/BWA_PIPELINE3_get_mapstats.pl)
echo "finished"
echo "$sample,$nofReads,$mReads,$umReads,$stats" >> $bamDir/sampleSet.mapStats

## Put the current date into a variable and report it before we exit.
GSITSENDDATE=`/bin/date`
echo "----------------------------------------------"
echo "**** JOB DONE, EXITING 0 AT $GSITSENDDATE"
##
## Exit with return code 0, otherwise the job would exit with the 
## return code of the failed ls command.
exit 0
##
########################################
