#!/usr/bin/perl -w
use Net::FTP;
## merges individual fastas into one multi-entry file, according to a supplied list of files for a specific project

### read in a paired list of the form [DB,taxID_file], where 'DB'is the name of the folder in U:\RefSets\LANDSCAPE_REF_GENOMES\ corresponding to a specific database, and taxID_file is a file listing the taxonIDs desired from that DB
### for each taxon, in a DB, 
### unzip corresponding file(keep original) and open each fasta (may be just 1 or >1),
### make sure its not a plasmid
### reformat header to match: [taxID_contigID_contigAccession; optional: originalFile, description]
### add sequence to merged file
### remove each

$num_args = $#ARGV + 1;
if ($num_args < 3) 
{
  print "\nUsage: LANDSCAPE_get_taxon_info.pl outfile DB1 taxID_list1 [DB2 taxID_list2 ...] \n";
  exit;
}

$f_info = shift @ARGV;
open (FH_INFO, ">$f_info") or die "Cannot open $f_info for writing.\n";
print FH_INFO "taxID\tacc\ttaxName\tlineage\tgenomeLength\tgeneCt\tdate\tscfAcc\tseqName\n";	
#open (FH_LOG, ">$f_merged.log") or die "Cannot open logfile for writing.\n";
	
while (@ARGV)
{	
	$DB = shift @ARGV;
	$f_taxIDs = shift @ARGV;
	
	if ($DB eq "NCBI_GENBANK")
	{	$ftp = 'ftp.ncbi.nih.gov/genbank/genomes/Bacteria';	}
	elsif ($DB eq "NCBI_BACTERIA_DRAFT")
	{	$ftp = 'ftp.ncbi.nih.gov/genomes/Bacteria_DRAFT';	}
	
	
#####################################################
# --- --- Get a list of all desired species --- --- #
	$nof_taxIDs = 0;
	print "getting info for: \n";
	%h_TaxIDs=();
	open (FH, "$f_taxIDs") or die "Cannot open $f_taxIDs for reading.\n";
	while(<FH>) {
		$line=$_; chomp($line); 
		$h_TaxIDs{$line}=0;
		print "$line|";
		$nof_taxIDs++;
	}
	close(FH);
	print "\n";
	
################################################
##list folders in the ftp directory
print "logging into $ftp ...\n";
($ftp_host, $ftp_dir) = ($ftp =~ m/([^\/]+)(\/.+)/); 
$ftp = Net::FTP->new($ftp_host) or die "cant connect to $@!\n";
$ftp->login("anonymous", 'greensi@uw.edu') or die "cant login ", $ftp->message;
$ftp->cwd($ftp_dir) or die "cant change dir! ", $ftp->message;
@ftp_folders = $ftp->ls();
$ftp->binary();		

###################################################################
## open each rpt file and see if it corresponds to a taxa of interest
foreach $spc_folder (@ftp_folders)
{
	@files = $ftp->ls($spc_folder . "\/*.rpt");
	
	foreach $rpt_file (@files)
	{	
		$taxID="";$acc="";$taxName="";$lineage="";$genomeLength="";$geneCt="";$date="";$scfAcc="";$seqName="";
		$temp_rpt = $ftp->get($rpt_file, "temp.rpt") or die "get failed ", $ftp->message;
		($acc) = ($rpt_file =~ m/([^\\\/]+).rpt$/);
		open (FH, "$temp_rpt") or die "Cannot open $temp_rpt for reading.\n";
		while(<FH>)
		{	
			$line = $_;	chomp $line;		
			if ($line =~ m/Taxname/)
			{	($rest,$taxName) = split(/: /,$line);	}
			if ($line =~ m/Taxid/)
			{	($rest,$taxID) = split(/: /,$line);	}
			if ($line =~ m/length/)
			{	($rest,$genomeLength) = split(/= /,$line);	}
			if ($line =~ m/Protein count/)
			{	($rest,$geneCt) = split(/: /,$line);	}
		}
		close(FH);
		
#####################################		
#### if it does, get the lineage, date, and sequenceName from the gbk file		
		if (exists $h_TaxIDs{$taxID})
		{	
			$numFound++;print "$numFound :$taxID\n";
			$gbk_file = $rpt_file; $gbk_file =~ s/rpt/gbk/;
			$temp_gbk = $ftp->get($gbk_file, "temp.gbk") or die "get failed ", $ftp->message;
			open (FH, "$temp_gbk") or die "Cannot open $temp_gbk for reading.\n";
			while(<FH>)
			{	
				$line = $_;	chomp $line;		
				if ($line =~ m/LOCUS/)
				{	@fields = split(/\s+/,$line);	
					$acc=$fields[1]; 
					$date=pop(@fields); 
				}
				if ($line =~ m/DEFINITION/)
				{	($seqName) = ($line =~ m/DEFINITION\s+(.+)$/);	print "$seqName\n";
					$line = <FH>; chomp $line;
					
					while ($line =~ /^\s/)
					{	$line =~ s/^\s+//;
						$line =~ s/\s+$//;
						$seqName .=$line;
						$line = <FH>; chomp $line;
					}
				
				}
				if ($line =~ m/ORGANISM/)
				{		
					$lineage = "";
					while ($lineage !~ m/\.$/)
					{	
						$line = <FH>;	chomp $line; 
						($line) = ($line =~ m/^\s+(\S.+\S)$/);
						$lineage .= $line;
					}
				}	
				if ($line =~ m/WGS_SCAFLD/)
				{	($scfAcc) = ($line =~ m/WGS_SCAFLD\s+(.+)$/);	}
									
				if ($line =~ m/FEATURES/ & ($seqName !~ m/shotgun/))
				{	last;	}
				
			}
			close(FH);
		#####################################		
		#### if not a plasmid, print to info_file		
			if ($seqName !~ m/plasmid/)
			{	print "---$acc:$taxName:$genomeLength:$geneCt:$date\n";
				print "---lineage:$lineage\n"; 
				print FH_INFO "$taxID\t$acc\t$taxName\t$lineage\t$genomeLength\t$geneCt\t$date\t$scfAcc\t$seqName\n";	}
		}
	}
}
$ftp->quit;
}
close(FH_INFO);
die;
# --- --- --- --- --- -- -- --- --- --- --- --- #
