#!/usr/bin/perl -w

## merges individual fastas into one multi-entry file, according to a supplied list of files for a specific project

### read in a paired list of the form [DB,taxID_file], where 'DB'is the name of the folder in U:\RefSets\LANDSCAPE_REF_GENOMES\ corresponding to a specific database, and taxID_file is a file listing the taxonIDs desired from that DB
### for each taxon, in a DB, 
### unzip corresponding file(keep original) and open each fasta (may be just 1 or >1),
### make sure its not a plasmid
### reformat header to match: [taxID_contigID_contigAccession; optional: originalFile, description]
### add sequence to merged file
### remove each

$num_args = $#ARGV + 1;
if ($num_args < 3) 
{
  print "\nUsage: LANDSCAPE_merge_project_fastas.pl mergedfile DB1 taxID_list1 [DB2 taxID_list2 ...] \[file_base\] \n";
  exit;
}

$f_merged = shift @ARGV;
open (FH_MERGED, ">$f_merged") or die "Cannot open $f_merged for writing.\n";
open (FH_HEADERS, ">$f_merged.headers") or die "Cannot open headersfile for writing.\n";
select((select(FH_HEADERS), $|=1)[0]);
open (LOG, ">$f_merged.log") or die "Cannot open logfile for writing.\n";
select((select(LOG), $|=1)[0]);
	
$dir_fastas = '/net/gs/vol1/home/greensi/RefSets/LANDSCAPE_REF_GENOMES/';

while (@ARGV)
{	
	$DB = shift @ARGV;
	$f_taxIDs = shift @ARGV;

	# --- --- Get a list of desired species from DB --- --- #
	$nof_taxIDs = 0;
	print LOG "getting records from $DB for: \n";
	print "getting records from $DB for: \n";
	%h_TaxIDs=();
	open (FH, "$f_taxIDs") or die "Cannot open $f_taxIDs for reading.\n";
	while(<FH>) {
		$line=$_; chomp($line); 
		$h_TaxIDs{$line}=0;
		print LOG "$line|";
		print "$line|";
		$nof_taxIDs++;
	}
	close(FH);
	print LOG "\n";
	print "\n";
	
	# --- --- Find file corresponding to each taxID, unzip and open each fasta --- --- #
	$nof_entries = 0;
	foreach $taxID (sort keys %h_TaxIDs)
	{	#print $dir_fastas . $DB . "/". $taxID . "*\n";
		@files = glob($dir_fastas . $DB . "/nucleotide_fastas/REVISION/" . $taxID . '_*' ); #print $dir_fastas . $DB . '/nucleotide_fastas/REVISION/' . $taxID . "_*\n"; 
		foreach $file (@files)
		{	print "unzipping  $file\n"; 
			if ($DB eq "NCBI_GENBANK")	
			{	system('tar', '-xzf', "$file"); 	}
			else
			{	system('tar', '-xzf', "$file", '-C', $dir_fastas . $DB . "/nucleotide_fastas/REVISION/" ); 	}
			
			
			@fastas = glob($dir_fastas . $DB . "/nucleotide_fastas/REVISION/" . "*.fna"); print "merging @fastas\n";	
			print "merging @fastas\n";
			
			foreach $fasta (@fastas)
			{	
				open (FH, $fasta) or die "Cannot open $fasta for reading.\n";
				while (<FH>)
				{
					$line = $_; chomp $line;
					if (substr($line,0,1) eq ">")
					{	$header = $line; 
						if ($header =~ m/plasmid/)
						{	$get_seq=0;	}
						else
						{	$get_seq=1;
							
							($acc) = ($fasta =~ m/([^\\\/]+).fna$/);
							$nof_entries++;
							$h_TaxIDs{$taxID}++;
							$new_header = ">" . $taxID . "_" .  $h_TaxIDs{$taxID} . "_" . $acc . " " . substr($header, 1, length($header)-1);
							print FH_MERGED "$new_header\n";
							print FH_HEADERS "$new_header\n";
							
						}
					}
					else
					{
						if ($get_seq)
						{	print FH_MERGED "$line\n";	}
					}
				} close(FH);
				system('rm',$fasta);
				
			}
		}
	}
	## PRINT COUNTS
	print LOG "************************************\n";
	print LOG "$nof_entries total contigs merged! \n";
	print LOG "nof contigs merged for each taxID:\n";
	print  "************************************\n";
	print  "$nof_entries total contigs merged! \n";
	print  "nof contigs merged for each taxID:\n";
	foreach $taxID (sort keys %h_TaxIDs)
	{	print LOG "$taxID:$h_TaxIDs{$taxID}|";	
		print "$taxID:$h_TaxIDs{$taxID}|";	
	}
	print LOG "************************************\n";
	print LOG "\n the following taxIDs had no valid contigs:\n";
	print  "************************************\n";
	print "\n the following taxIDs had no valid contigs:\n";

	foreach $taxID (sort keys %h_TaxIDs)
	{	if ($h_TaxIDs{$taxID}<1)
		{	print LOG "$taxID|";
			print "$taxID|";
		}
	}
	print LOG "\n";
	print "\n";

}
close(FH_MERGED);
close(FH_HEADERS);	
close(LOG);					