#!/usr/bin/perl -w

## merges genes from individual fastas into one multi-entry file, according to a supplied list of taxon iDs for a specific project

### read in a paired list of the form [DB,taxID_file], where 'DB'is the name of the folder in U:\RefSets\LANDSCAPE_REF_GENOMES\ corresponding to a specific database, and taxID_file is a file listing the taxonIDs desired from that DB
### for each taxon, in a DB, 
### unzip corresponding file(keep original) and open each fasta (may be just 1 or >1),
### make sure its not a plasmid
### open corresponding gene file
### extract each gene sequence and create header: [geneID_contigID_start_stop; optional: geneName, geneLength]
### add sequence to merged file
### remove each

$num_args = $#ARGV + 1;
if ($num_args < 3) 
{
  print "\nUsage: LANDSCAPE_merge_genes_by_taxon.pl mergedfile DB1 taxID_list1 [DB2 taxID_list2 ...] \[KOfile\] \n";
  exit;
}

$f_merged = shift @ARGV;
open (FH_MERGED, ">$f_merged") or die "Cannot open $f_merged for writing.\n";
open (FH_HEADERS, ">$f_merged.headers") or die "Cannot open headersfile for writing.\n";
select((select(FH_HEADERS), $|=1)[0]);
open (LOG, ">$f_merged.log") or die "Cannot open logfile for writing.\n";
select((select(LOG), $|=1)[0]);
open (FH_BED, ">$f_merged.BED") or die "Cannot open BEDfile for writing.\n";
select((select(FH_BED), $|=1)[0]);

	
$dir_fastas = '/net/gs/vol1/home/greensi/RefSets/LANDSCAPE_REF_GENOMES/';

while (@ARGV)
{	
	$DB = shift @ARGV;
	$f_taxIDs = shift @ARGV;

	# --- --- Get a list of desired species from DB --- --- #
	$nof_taxIDs = 0; $nofGenes = 0;
	print LOG "getting records from $DB for: \n";
	print "getting records from $DB for: \n";
	%h_taxID2contigs=();
	%h_taxID2genes=();
	open (FH, "$f_taxIDs") or die "Cannot open $f_taxIDs for reading.\n";
	while(<FH>) {
		$line=$_; chomp($line); 
		$h_taxID2contigs{$line}=0;
		$h_taxID2genes{$line}=0;
#		print LOG "$line|";
		print "$line|";
	}
	close(FH);
	print "\n";
#	print LOG "\n";
	
	# --- --- Find file corresponding to each taxID, unzip and open each fasta --- --- #
	
	foreach $taxID (sort keys %h_taxID2contigs)
	{	
		$nof_taxIDs++;	print "$nof_taxIDs:$taxID\n";
		@files = glob($dir_fastas . $DB . '/nucleotide_fastas/' . $taxID . '_*' ); 
		foreach $file (@files)
		{	
			$ptt_file = $file; 
			$ptt_file =~ s/fna/ptt/;
			$ptt_file =~ s/nucleotide_fastas/gene_calls/; 
			
			system('tar', '-xzf', "$file", '-C',$dir_fastas);
			system('tar', '-xzf', "$ptt_file", '-C',$dir_fastas);
		
			@fastas = glob($dir_fastas  . "*.fna");
			
			foreach $fasta (@fastas)
			{	
				($acc) = ($fasta =~ m/([^\\\/]+).fna$/);
				
				
				
				$ptt = $fasta; $ptt =~ s/fna/ptt/;$ptt =~ s/nucleotide_fastas/gene_calls/;
				open (FH, $fasta) or die "Cannot open $fasta for reading.\n";
				$line = <FH>; chomp $line;
				
				if (substr($line,0,1) eq ">")
				{	$header = $line; 	}
				else {	die "no header! $fasta\n";	close(FH); }
				
				if ($header =~ m/plasmid/)
				{	print "ignoring plasmid sequence: $header\n";	close(FH); }
				else
				{	$h_taxID2contigs{$taxID}++;
					$seq="";
					while (<FH>)
					{	$line =$_; chomp($line); 
						$seq .= $line;	
					}
					close(FH); 
					
					
					$no_genes=0;
					open (PTT, $ptt) or $no_genes=1;
					if (! $no_genes)
					{
					$line=<PTT>;	#print "$line\n";
					$line=<PTT>;  chomp($line);
					($nof_orfs) = ($line =~ m/^(\S+)/);
					<PTT>;
					while(<PTT>)
					{	
						$line=$_; chomp($line);
						($loc, $strand, $length, $pid, $gene, $synonym, $code, $cog, $product) = split(/\t/,$line);
						($start,$end) = ($loc =~ m/(.+)\.\.(.+)/);
						if (length($seq)>=$end)
						{
							$orf_seq = substr($seq,$start-1,$end-$start+1);
							if ($strand eq "-")
							{  $orf_seq = scalar reverse $orf_seq; 
								$orf_seq=~ s/G/c/g; 
								$orf_seq=~ s/C/g/g;
								$orf_seq=~ s/A/t/g;
								$orf_seq=~ s/T/a/g;
								$orf_seq=~ s/g/G/g; 
								$orf_seq=~ s/c/C/g;
								$orf_seq=~ s/t/T/g;
								$orf_seq=~ s/a/A/g;
							}
							
							$h_taxID2genes{$taxID}++; $nofGenes++;
							$myGeneID = "G" . $taxID . "-" . $h_taxID2genes{$taxID};
#							$geneHeader = $myGeneID	. "_C" . $h_taxID2contigs{$taxID} . "_" . $start . "_" . $end . " length:$length $synonym $product";
							$geneHeader = $myGeneID	. "_C" . $h_taxID2contigs{$taxID} . "_" . $start . "_" . $end . " length:$length pid:$pid $synonym $product";
							$contig_header = $taxID . "_" .  $h_taxID2contigs{$taxID} . "_" . $acc;
							
							print FH_HEADERS ">$geneHeader\n";
							print FH_MERGED ">$geneHeader\n";
							print FH_MERGED "$orf_seq\n";
							print FH_BED "$contig_header\t$start\t$end\t$myGeneID\t$length\t$strand\t$pid\t$synonym\t$product\n";
						}
						else
						{	print "gene outside sequence range! taxon:$taxID contig:$h_taxID2contigs{$taxID} gene: $h_taxID2genes{$taxID},$synonym contigLength: " . length($seq) . " geneEnd: $end\n";	}
					}
					close(PTT);	 system('rm',$ptt);
				}
				else
				{	print "no gene calls for $taxID:$ptt!\n"; 	}
				}
				system('rm',$fasta);
			}
		}
	}
	
## PRINT COUNTS
	print LOG "************************************\n";
	print LOG "$nofGenes total genes merged! \n";
	print LOG "nof genes merged for each taxID:\n";
	print  "************************************\n";
	print  "$nofGenes total genes merged! \n";
	print  "nof genes merged for each taxID:\n";
	foreach $taxID (sort keys %h_taxID2genes)
	{	print LOG "$taxID:$h_taxID2genes{$taxID}|";	
		print "$taxID:$h_taxID2genes{$taxID}|";	
	}
	print LOG "************************************\n";
	print LOG "\n the following taxIDs had no valid contigs:\n";
	print  "************************************\n";
	print "\n the following taxIDs had no valid contigs:\n";

	foreach $taxID (sort keys %h_TaxIDs)
	{	if ($h_taxID2genes{$taxID}<1)
		{	print LOG "$taxID|";
			print "$taxID|";
		}
	}
	print LOG "\n";
	print "\n";
}
close(FH_MERGED);
close(FH_HEADERS);
close(BED);		
close(LOG);			