#!/usr/bin/perl -w
use Net::FTP;

## downloads complete genomes from NCBI according to a supplied list of taxonIDs

###READ IN A LIST OF TAXON IDS, AND AN FTP ADDRESS
###FOR EACH FOLDER IN FTP
###DOWNLOAD EACH .RTP FILE (not .rtp.tgz!), LOOK INSIDE AT TAXON ID, THEN REMOVE .RPT
### IF TAXON ID IS ON THE LIST, DOWNLOAD .FNA FILE
### SAVE AS taxID.fileName.fasta
### gzip it, if not already

#######################################################
###          FILES                                 ###
######################################################
$LAB="/net/borenstein/vol1";

## NCBI GENBANK ftp site
$taxID_list = $LAB . '/PROJECTS/StrainVariation_SG/DATA/REVISION/new_ref_genomes/newGenomes.GENBANK.taxids'; ##list of taxon IDs
$ftp = 'ftp.ncbi.nih.gov/genbank/genomes/Bacteria';  
$fasta_outDir = 'RefSets/LANDSCAPE_REF_GENOMES/NCBI_GENBANK/protein_fastas/';
#get_fastas();

## NCBI BACTERIA_DRAFT ftp site
$taxID_list = $LAB . '/PROJECTS/StrainVariation_SG/DATA/REVISION/new_ref_genomes/newGenomes.DRAFT.taxids'; ##list of taxon IDs
$ftp = 'ftp.ncbi.nih.gov/genomes/Bacteria_DRAFT';  
$fasta_outDir = 'RefSets/LANDSCAPE_REF_GENOMES/NCBI_BACTERIA_DRAFT/protein_fastas/REVISION/';
get_fastas();
#####################################################

sub get_fastas
{
# --- --- Get a list of all desired species --- --- #
$nof_taxIDs = 0;
print "getting records for: \n";
%h_TaxIDs=();
open (FH, "$taxID_list") or die "Cannot open $taxID_list for reading.\n";
while(<FH>) {
  $line=$_; chomp($line); 
  $h_TaxIDs{$line}=0;
  print "$line|";
  $nof_taxIDs++;
}
close(FH);
print "\n";
################################################
##list folders in the ftp directory
print "logging into $ftp ...\n";
($ftp_host, $ftp_dir) = ($ftp =~ m/([^\/]+)(\/.+)/); 
$ftp = Net::FTP->new($ftp_host) or die "cant connect to $@!\n";
$ftp->login("anonymous", 'greensi@uw.edu') or die "cant login ", $ftp->message;
$ftp->cwd($ftp_dir) or die "cant change dir! ", $ftp->message;
@ftp_folders = $ftp->ls();
$ftp->binary();

###################################################################
## open each rpt file and see if it corresponds to a taxa of interest
foreach $spc_folder (@ftp_folders)
{
	@files = $ftp->ls($spc_folder . "\/*.rpt");
	
	foreach $rpt_file (@files)
	{	
		$taxID="";$acc="";$taxName="";$lineage="";$genomeLength="";$geneCt="";$date="";$scfAcc="";$seqName="";	
		$temp_rpt = $ftp->get($rpt_file, "temp.rpt") or die "get failed ", $ftp->message;
		($acc) = ($rpt_file =~ m/([^\\\/]+).rpt$/);
		open (FH, "$temp_rpt") or die "Cannot open $temp_rpt for reading.\n";
		while(<FH>)
		{	
			$line = $_;	chomp $line;		
			if ($line =~ m/Taxname/)
			{	($rest,$taxName) = split(/: /,$line);	}
			if ($line =~ m/Taxid/)
			{	($rest,$taxID) = split(/: /,$line);	}
			if ($line =~ m/length/)
			{	($rest,$genomeLength) = split(/= /,$line);	}
			if ($line =~ m/Protein count/)
			{	($rest,$geneCt) = split(/: /,$line);	}
		}
		close(FH);
		
		
			$gbk_file = $rpt_file; $gbk_file =~ s/rpt/gbk/;
					$temp_gbk = $ftp->get($gbk_file, "temp.gbk") or die "get failed ", $ftp->message;
					open (FH, "$temp_gbk") or die "Cannot open $temp_gbk for reading.\n";
					while(<FH>)
					{	
						$line = $_;	chomp $line;		
						if ($line =~ m/LOCUS/)
						{	@fields = split(/\s+/,$line);	
							$acc=$fields[1]; 
							$date=pop(@fields); 
						}
						if ($line =~ m/DEFINITION/)
						{	($seqName) = ($line =~ m/DEFINITION\s+(.+)$/);	print "$seqName\n";
							$line = <FH>; chomp $line;
							
							while ($line =~ /^\s/)
							{	$line =~ s/^\s+//;
								$line =~ s/\s+$//;
								$seqName .=$line;
								$line = <FH>; chomp $line;
							}
						
						}
						if ($line =~ m/ORGANISM/)
						{		
							$lineage = "";
							while ($lineage !~ m/\.$/)
							{	
								$line = <FH>;	chomp $line; 
								($line) = ($line =~ m/^\s+(\S.+\S)$/);
								$lineage .= $line;
							}
						}	
						if ($line =~ m/WGS_SCAFLD/)
						{	($scfAcc) = ($line =~ m/WGS_SCAFLD\s+(.+)$/);	}
											
						if ($line =~ m/FEATURES/ & ($seqName !~ m/shotgun/))
						{	last;	}
						
					}
					close(FH);
				#####################################		
		
#####################################		
#### if it does, download the fasta and ptt file
		if (exists $h_TaxIDs{$taxid})
		{	
			$numFound++;print "$numFound :";
			@fastas = $ftp->ls($spc_folder . "\/$acc*.faa*");
			foreach $fasta_file (@fastas)
			{	($filename) = ($fasta_file =~ m/([^\/]+)$/);
				$new_fasta = $fasta_outDir . $taxid . "_" . $filename;
				$ftp->get($fasta_file, $new_fasta) or die "get failed ", $ftp->message;
				if ( $new_fasta !~ m/.tgz$/)
				{	system('tar','czf',$new_fasta . ".tgz", $new_fasta); $new_fasta = $new_fasta . ".tgz";}
				print " $new_fasta\n";	
				$h_TaxIDs{$taxid}++;
			}
		}
	}
}
$ftp->quit;
##########################################
#### print file counts to log file

open (FH, ">$fasta_outDir" . "log.txt") or die "Cannot open logfile for writing.\n";
print FH "TAXID\tNOF_FILES\n";
foreach $taxid (sort keys %h_TaxIDs)
{	print FH "$taxid\t$h_TaxIDs{$taxid}\n";	}
close(FH);
}	
die;
# --- --- --- --- --- -- -- --- --- --- --- --- #


